/*
 PROJECT: MySensors - Servo sketch, translates Pitch and Yaw to pan/tilt servo
 PROGRAMMER: AWI
 DATE: december 26, 2015/ last update: december 26, 2015
 FILE: AWI_pan_tilt_servo.ino
 LICENSE: Public domain

 Hardware: any MySensors supported ATMega328p board w/ NRF24l01
	and MySensors 1.6 (Development)
		
Special:
	
	
Summary:
	Double (pan/tilt) servo and PWM Led
	
Remarks:
	Fixed node-id
	
Change log:
	20151226 - initial sketch
	20160106 - update with pwm led
	20160322 - update with V_ORIENTATION (=V_TEXT) type
*/

// Enable debug prints to serial monitor
#define MY_DEBUG 
#define MY_RADIO_NRF24										// Enable and select radio type attached
#define MY_NODE_ID 51
// #define MY_RF24_CE_PIN 7									// Ceech board, 3.3v (7,8)  (pin default 9,10)
// #define MY_RF24_CS_PIN 8
// #define DESTINATION_NODE 51								// node to send position info to (default 0 = gateway)
// Enabled repeater feature for this node
#define MY_REPEATER_FEATURE

#define V_ORIENTATION V_TEXT								// Own (not existing in MySensors protocol) type for orientation (payload = Pitch ; Roll ; Yaw in degrees)
#define S_ORIENTATION S_INFO								// Yaw = 0 (north)to 360) ; Horizontal = 0 ; 

#include <SPI.h>
#include <MySensor.h> 
#include <Servo.h>
#include <FastLED.h>										// https://github.com/FastLED/FastLED

//***** MySensors settings
#define orientationChild 1									// new (non-existing) sensor for orientation (Yaw Pitch Roll)
#define panChild 2											// traditional "level" type for servo pan (Yaw)
#define tiltChild 3											// traditional "level" type for servo tilt (Pitch)
#define ledChild 4											// PWM led

//***** Pin assignments
const int yawServoPin = 4 ;									// yaw servo
const int pitchServoPin = 7 ;								// pitch servo
const int pwmLedPinR = 3 ;									// pwm led (0..255)
const int pwmLedPinG = 5 ;									// pwm led (0..255)
const int pwmLedPinB = 6 ;									// pwm led (0..255)


//***** Servo's
Servo servoH, servoV;										// Horizontal (Yaw) and vertical(Pitch) servo's

//***** PWM leds
CRGB curRGB = 0xFFDEAD ;							// PWM led (use Fastled datatype for convenience)


//***** 
const unsigned long displayInterval = 50000UL ;				// display update in ms
const unsigned long servoSettleDelay = 2000 ;				// settle time for servo ()
volatile int Pitch, Roll, Yaw ; 							// storage of current values
volatile int curBrightness = 0 ;							// brightnesslevel
unsigned long lastServoUpdate, lastDisplayUpdate = 0 ;		// last update for loop timers
boolean servoAttached ;										// flag indicates servo powered state


void setup()
{
	// set servos (bw)	
	attachServos(true) ;									// power up sensors on init ;
	// init brightnesslevel
	analogWrite(pwmLedPinR, curBrightness ) ;				// could be remembered in EEPROM
	analogWrite(pwmLedPinG, curBrightness ) ;				// could be remembered in EEPROM
	analogWrite(pwmLedPinB, curBrightness ) ;				// could be remembered in EEPROM
	}

void presentation() {
  // Register binary input sensor to gw (they will be created as child devices)
  // You can use S_DOOR, S_MOTION or S_LIGHT here depending on your usage. 
  // If S_LIGHT is used, remember to update variable type you send in. See "msg" above.
	sendSketchInfo("Pan Tilt servo 51", "1.0");
	present(orientationChild, S_ORIENTATION, "YawPitchRoll") ;  
	present(panChild, S_DIMMER, "Pan") ;  
	present(tiltChild, S_DIMMER, "Tilt") ;  
	present(ledChild, S_RGB_LIGHT, "PWM RGB led") ;  
}

void loop()
{  
    unsigned long now = millis();
	if (now >= lastDisplayUpdate + displayInterval){
		Serial.println("Servo position");
		Serial.print("Yaw  : ");
		Serial.println( Yaw );
		Serial.print("Pitch: ");
		Serial.println( Pitch );
		Serial.print("Roll : ");
		Serial.println( Roll);
		Serial.println();
		lastDisplayUpdate = now ;
		}
	if (now >= lastServoUpdate + servoSettleDelay){				// check if time for power down of servo's 
		attachServos(false) ;
		}
}

// Handle incoming messages 
void receive(const MyMessage &message) {  						// Expect few types of messages from controller
	Serial.print("Last reading for sensor: ");
	Serial.print(message.sensor);                
	Serial.print(", Message: ");
	Serial.println(message.getString());
	switch (message.sensor){
		case orientationChild:
			if (message.type==V_ORIENTATION) {
				char payload[12] ;								// temporary buffer
				strcpy(payload, message.getString());			// read and copy payload
				Yaw = atoi(strtok(payload, ";"));				// get variables from payload (; seperated int)
				Pitch = atoi(strtok(NULL, ";"));
				Roll = atoi(strtok(NULL, ";"));
				attachServos(true) ;							// power up servo's
				servoH.write(map(-constrain(Yaw, -90, 90),-90, 90, 0, 180));		// servo middle 90 deg == 0 deg Yaw 
				servoV.write(map(Pitch, -90, 90, 0, 180));
				}
			break ;
		case panChild:
			if (message.type==V_PERCENTAGE) {
				attachServos(true);
				servoH.write(map(message.getInt(),0, 100, 0,180));
				}
			break ;
		case tiltChild:	
			if (message.type==V_PERCENTAGE) {
				attachServos(true);
				servoV.write(map(message.getInt(),0, 100, 0,180));
				}
			break ;
		case ledChild:
			if (message.type == V_PERCENTAGE){						// dimmer message
				curBrightness = message.getInt();
				analogWrite(pwmLedPinR, curRGB.red * curBrightness / 100 );
				analogWrite(pwmLedPinG, curRGB.green * curBrightness / 100 );
				analogWrite(pwmLedPinB, curRGB.blue * curBrightness / 100 );
			} else if (message.type == V_RGB){								// color message
				curRGB = strtol( message.getString(), NULL, 16) ;		// read payload and convert to long RGB value
				analogWrite(pwmLedPinR, curRGB.red );
				analogWrite(pwmLedPinG, curRGB.green );
				analogWrite(pwmLedPinB, curRGB.blue );
			} else if (message.type == V_STATUS) {				// switch between off and currentbrightness
				analogWrite(pwmLedPinR, (message.getInt()==0)?0:curRGB.red * curBrightness / 100 );
				analogWrite(pwmLedPinG, (message.getInt()==0)?0:curRGB.green * curBrightness / 100 );
				analogWrite(pwmLedPinB, (message.getInt()==0)?0:curRGB.blue * curBrightness / 100 );
			}
			Serial.print("PWM level: ") ;
			Serial.println(curBrightness) ;
			break ;
	}
}

// attach/ detach servo
void attachServos(bool state)
{
	if (state){													// attach if true
		servoH.attach(yawServoPin, 790, 2250); 					// servo control pin, and range (needs calibration)
		servoV.attach(pitchServoPin, 920, 2250); 				//servo control pin, and range if desired
		lastServoUpdate = millis();								// start timer
	} else {
		servoH.detach();
		servoV.detach();
	}
	servoAttached = state ;
}
